<?php

namespace SM\Util;

use Teknisa\Libs\Service\CRUD as CRUDService;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\DateUtil;

class Utilities {
    
    protected $crudService;
    protected $environment;
    protected $address;
    protected $confirmaEnderecoService;
    const SERVICES_ENTITY_DIR = '\Teknisa\Model\Services\Entities\\';
    public function __construct(CRUDService $crudService, Environment $environment, $address, $confirmaEnderecoService) {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->address = $address;
        $this->confirmaEnderecoService = $confirmaEnderecoService;
    }
    
    /**
	 * Valida no banco se o dados que estao sendo inseridos ja não existem nos campos dos quais estão sendo gravados
	 * 
	 * Retona true se nao existir ou false se já existir 
	 * 
	 * @param string  $string
	 * @param string  $string
     * @param Array  $string
	 */
    public function validateDuplicatedRecords($row, $dataSourceName, $pkColumns, $validationColumns) {
        $conditions = array();

        foreach ($validationColumns as $column) {
            $value = $row[$column];
            array_push($conditions, array(
                "columnName" => $column,
                "operator"   => is_array($value) ? "IN" : ((!empty($value)) ? '=' : "IS_NULL"),
                "value"      => $value
            ));
        }
        
        if (!in_array("NRORG", $validationColumns)) {
            array_push($conditions, array(
                "columnName" => "NRORG",
                "operator"   => "=",
                "value"      => $this->environment->getNrOrg()
            ));
        }
        
        if ($pkColumns) {
            foreach ($pkColumns as $column) {
                $value = $row[$column];
                array_push($conditions, array(
                    "columnName" => $column,
                    "operator"   => "<>",
                    "value"      => $value ? $value : -1
                ));
            }
        }
        
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
        if (!empty($dataSet->getRows())) {
           return false;
        }
        else {
            return true;
        }    
        
    }
    
    /**
     * Recebe uma string com valores separados por um delimitador, e retorna um array
     * a partir dessa string.
     * 
     * @param string  $valor
     * @param boolean $int
     * @param string  $delimitador
     */
    public static function getArrayDeString($valor, $int = false, $delimitador = ",") {
        if (empty($valor)) {
            $value = array();
        }
        else {
            if ($int) {
                $value = array_map("intval", explode($delimitador, $valor));
            }
            else {
                $value = explode($delimitador, $valor);
            }
        }
        
        return $value;
    }
    
    /**
     * Recebe um objeto DateTime e retorna o nome do mês em português.
     */
    public static function getNomeMesPortugues($data) {
        $mes = $data->format("m");
        $nome = null;
        
        switch ($mes) {
            case "01": $nome = "Janeiro";
                break;
                
            case "02": $nome = "Fevereiro";
                break;
                
            case "03": $nome = "Março";
                break;
                
            case "04": $nome = "Abril";
                break;
                
            case "05": $nome = "Maio";
                break;
                
            case "06": $nome = "Junho";
                break;
                
            case "07": $nome = "Julho";
                break;
                
            case "08": $nome = "Agosto";
                break;
                
            case "09": $nome = "Setembro";
                break;
                
            case "10": $nome = "Outubro";
                break;
                
            case "11": $nome = "Novembro";
                break;
                
            case "12": $nome = "Dezembro";
                break;
            
            default  : $nome = null;
        };
        
        return $nome;
    }
    
    /**
     * Formata um número para a exibição.
     * 
     * @param integer $numero
     * @param integer $casasDecimais
     * 
     * @return string
     */
    public static function formatarNumero($numero, $casasDecimais = 2, $separadorDecimal = ",", $separadorMilhar = ".") {
        if (!is_numeric($numero)) {
            $numero = 0.0;
        }
        return number_format($numero, $casasDecimais, $separadorDecimal, $separadorMilhar);
    }
    
    /**
     * Aplica a máscara de CPF ou CNPJ em uma string.
     * 
     * @param string $str
     */
    public static function formatarCpfCnpj($str) {
       if (strlen($str) == 14) {
           return self::mascararString("##.###.###/####-##", $str);
       }
       else if (strlen($str) == 11) {
           return self::mascararString("###.###.###-##", $str);
       }
       else {
           return $str;
       }
    }
    
    /**
     * Aplica a máscara de CEP em uma string.
     * 
     * @param string $str
     */
    public static function formatarCep($str) {
       if (strlen($str) == 8) {
           return self::mascararString("##.###-###", $str);
       }
       else {
           return $str;
       }
    }
    
    /**
     * Aplica uma máscara a uma string.
     * 
     * @param string $mask Máscara em "#" (ex: CPF: ###.###.###-##)
     * @param string $str String a ser formatada
     */
    public static function mascararString($mask, $str) {
        $str = str_replace(" ", "", $str);
    
        for ($i=0; $i < strlen($str); $i++) {
            $mask[strpos($mask,"#")] = $str[$i];
        }
    
        return $mask;
    }
    
    public static function uploadArquivoFileServer($zhFileUploader, $customFolder, $filePath) {
        if (is_file($filePath)) {
            $size = filesize($filePath);
            $fileName = basename($filePath);
            $fileContent = file_get_contents($filePath);
            $type = null;
            
            switch(strtolower(substr(strrchr($fileName,"."), 1))) {
                case "pdf": {
                    $type = "application/pdf"; break;
                }    
                case "docx": {
                    $type = "application/vnd.openxmlformats-officedocument.wordprocessingml.document"; break;
                }    
                case "gif": {
                    $type = "image/gif"; break;
                }    
                case "png": {
                    $type = "image/png"; break;
                }    
                case "jpg": {
                    $type = "image/jpg"; break;
                }    
                case "txt": {
                    $type = "text/plain"; break;
                }
            }
            
            return self::uploadFileServer($zhFileUploader, $customFolder, $fileName, $fileContent, $size, $type);
        }
        else
            return null;
    }
    
    public static function uploadFileServer($zhFileUploader, $customFolder, $fileName, $fileContent, $size, $type) {
        $base64Content = base64_encode($fileContent);
        
        $fileData = array(
            'b64File' => 'data:'.$type.';base64,'.$base64Content,
            'name' => $fileName,
            'size' => $size,
            'type' => $type,
            'webkitRelativePath' => ''
        );

        $row = new \Zeedhi\Framework\DTO\Row(array('FILE' => array($fileData)));
        $zhFileUploader->uploadFilesInRow($row, 'FILE', $customFolder);
        
        return $row["FILE"][0];
    }
    
    public function findAddress(Row $row, Response $response){
        $this->address->findAddress($row, $response);
        if(!empty($response->getDataSets())){
            $foundAddress = $response->getDataSets()[0]->getRows();
            $foundAddress = $this->confirmaEnderecoService->confirmAddress($foundAddress);
            if(!empty($foundAddress['MESSAGES'])){
                $response->addMessage(new Message($foundAddress['MESSAGES']));        
            }
            
            $response->addDataSet(new DataSet('ENDERECO', $foundAddress['ADDRESS']));        
        }
    
    }

    public static function setDataSetDefault(Request $request, Response $response, array $dataSet)
    {
        $response->addDataset(new DataSet(explode('/', $request->getRoutePath())[1], $dataSet));
    }
    
    /**
     * Converta parte hora em decimal
     * @static
     *
     * @param $hora
     *
     * @return float
     */
    public static function converteHoraDecimal($hora) {
        $aux = floor($hora);

        if ($aux === $hora){
            return $hora;
        } else {
            $minuto = $hora - $aux;
            $decimal = $minuto * 100.0 / 60.0;
            $aux += $decimal;

            return $aux;
        }
    }

    public static function adicionarMes($data, $qtdMeses) {
        $novaData = DateUtil::adicionaIntervalo($data, $qtdMeses, DateUtil::MES);
        if ($data->format("m/Y") == $novaData->format("m/Y")) {
            $novaData = DateUtil::adicionaIntervalo($novaData, 3);
            $novaData = DateUtil::getPrimeiroDiaDoMes($novaData);
        }

        return $novaData;
    }

    public static function getCondicaoIn999(array $itens, $nomeColuna, $isString = false) {
        if (count($itens) > 999)
            $novoArray = array_chunk($itens, 999);
        else
            $novoArray = array($itens);

        $itensString = array();
        foreach ($novoArray as $value) {
            if ($isString)
                $itensString[] = "'" . implode("','", $value) . "'";
            else
                $itensString[] = implode(",", $value);
        }

        $primeiroInElementos = $itensString[0];
        unset($itensString[0]);
        
        $condicao = " ($nomeColuna IN ($primeiroInElementos) ";
        foreach ($itensString as $item) {
            $condicao .= " 
            OR $nomeColuna IN ($item) ";
        }

        return $condicao . ") ";
    }
    
    /**
     * Converte um número do formato string para float.
     * Pode estar tanto no formado americano quanto brasileiro.
     * 
     * @param $numero
     * @return float
     */
    public static function stringToFloat($numero) {
        if (is_string($numero) && strlen($numero) > 0) {
            $numero = str_replace(",", ".", $numero);
            $pos = strrpos($numero, ".");
            if ($pos !== false) {
                $numero = str_replace(".", "", substr($numero, 0, $pos)) . substr($numero, $pos);
                $numero = floatval($numero);
            }
        }
        
        return $numero;
    }
    
    /**
     * Gera o template teknisa com a mensagem do email
    **/
    public static function getTemplateEmailTeknisa(array $dadosSubstituiTemplate){
        $template = new \Teknisa\SM\Util\Mail\Templates\Teknisa\TemplateTeknisa();
        return $template->setValoresEmail($dadosSubstituiTemplate);
    }

    public static function downloadArquivoFileServer($fileUrl, $fileName, $fileType, $fileSize) {
        header('Accept-Ranges: none');
        header('Content-Type: ' . $fileType);
        //header('Content-Length: ' . $fileSize);
        header('Content-Disposition: inline; filename="' . $fileName . '"');
        
        ob_clean(); flush();
        readfile($fileUrl);
        
        exit;
    }

    public static function decodificarDynamicFilter($dynamicFilter)
    {
        return empty($dynamicFilter) ? null : json_decode(utf8_encode(base64_decode($dynamicFilter)), true);
    }
}