import { Utilities } from '@/util/Utilities';
import {
    Modal, ModalService, Select, Text, TextInput, Number, FileInput, Form, LoadingService, DialogService, Button,
} from '@zeedhi/common';
import {
    Config,
    Http,
    IEventParam,
    Loader, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Sm00001001TabAnexosDocumentoPropostaController {
    private basicModal!: Modal;

    public gridDocumentoProposta = Loader.getInstance('Sm00001001DocumentoPropostaController');
    
    async exibirModalAnexoDocumentoProposta() {
        this.basicModal = await ModalService.createFromJson('sm_00001_001_documento_proposta/modal/sm_00001_001_anexo_documento_proposta', true);
        this.basicModal.show();
    }

    public fecharModalAnexoDocumentoProposta() {
        this.basicModal.hide();
        this.basicModal?.destroy();
    }

    public async abrirModalAnexoDocumentoProposta() {
        await this.exibirModalAnexoDocumentoProposta();
        Metadata.getInstance<Text>('titleText').text = 'Novo Anexo';
    }

    public async abrirModalEdicaoAnexoDocumentoProposta({ row }: any) {
        await this.exibirModalAnexoDocumentoProposta();
        Metadata.getInstance<Text>('titleText').text = 'Edição de Anexo';
        Metadata.getInstance<TextInput>('NRORCAMENTODOCIDIOMA').value = row.NRORCAMENTODOCIDIOMA;
        Metadata.getInstance<TextInput>('NRSEQORCADOC').value = row.NRSEQORCADOC;
        Metadata.getInstance<Select>('CDIDIOM').value = row.CDIDIOM;
        Metadata.getInstance<TextInput>('IDPADRAO').value = row.IDPADRAO;
        Metadata.getInstance<Select>('IDATIVO').value = row.IDATIVO;
        Metadata.getInstance<TextInput>('NMARQUIVO').value = row.NMARQUIVO;
        Metadata.getInstance<TextInput>('DSCAMINHOARQ').value = row.DSCAMINHOARQ;
        Metadata.getInstance<TextInput>('DSTIPO').value = row.DSTIPO;
        Metadata.getInstance<Number>('VRTAMANHO').value = row.VRTAMANHO;

        if (Metadata.getInstance<TextInput>('NMARQUIVO').value != null) {
            Metadata.getInstance<FileInput>('ARQUIVO').isVisible = false;
        } else {
            Metadata.getInstance<FileInput>('ARQUIVO').isVisible = true;
        }
    }

    public async inserirAnexoDocumentoProposta() {
        const form = Metadata.getInstance<Form>('form_anexo_documento_proposta');
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-anexos-documento-proposta');
        if (Utilities.validateForm(form)) {
            const file = await Utilities.buildArrayToUpload('ARQUIVO');
            LoadingService.show();
            const params = {
                NRORCAMENTODOCIDIOMA: Metadata.getInstance<TextInput>('NRORCAMENTODOCIDIOMA').value,
                NRSEQORCADOC: this.gridDocumentoProposta.currentRow.NRSEQORCADOC,
                CDIDIOM: Metadata.getInstance<Select>('CDIDIOM').value,
                IDPADRAO: Metadata.getInstance<Select>('CDIDIOM').selectValue.IDPADRAO,
                IDATIVO: this.gridDocumentoProposta.currentRow.IDATIVO,
                NMARQUIVO: Metadata.getInstance<TextInput>('NMARQUIVO').value,
                DSCAMINHOARQ: Metadata.getInstance<TextInput>('DSCAMINHOARQ').value,
                DSTIPO: Metadata.getInstance<TextInput>('DSTIPO').value,
                VRTAMANHO: Metadata.getInstance<Number>('VRTAMANHO').value,
                ARQUIVO: file,
            };
            Http.post('/inserirAnexoDocumentoProposta', params).then(() => {
                this.fecharModalAnexoDocumentoProposta();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
                if (error.response.data.error === 'EXISTE') {
                    this.showErrorDialogValidacaoAnexoProposta();
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showErrorDialogValidacaoAnexoProposta() {
        DialogService.show({
            name: 'error-dialog-validacao',
            type: 'error',
            title: 'Atenção',
            text: 'Não é possível adicionar outro registro do mesmo idioma para o mesmo documento de contrato',
        });
    }

    public cliqueCelula({ column }: any) {
        let preventRowClick = false;
        if (column.name === 'DELETARANEXO' || column.name === 'DOWNLOADANEXO' || column.name === 'ATUALIZARANEXO') {
            preventRowClick = true;
        }
        return preventRowClick;
    }

    public habilitarBotaoGrid({ row }: any) {
        return (row.NMARQUIVO != null);
    }
    
    public downloadAnexoModeloDocumentoProposta({ row }: any) {
        const nrorcamentodocidioma = row.NRORCAMENTODOCIDIOMA;
        window.open(`${Config.endPoint}downloadAnexoModeloDocumentoProposta?nrorcamentodocidioma=${nrorcamentodocidioma}`);
    }

    public dialoglimparAnexoModeloDocumentoProposta({ row }: IEventParam<Button>) {
        DialogService.show({
            name: 'dialog-deletar-anexo-proposta',
            type: 'warning',
            title: 'Atenção',
            text: 'Deseja realmente excluir o documento anexado?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.limparAnexoModeloDocumentoProposta(row);
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public limparAnexoModeloDocumentoProposta(row: any) {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-anexos-documento-proposta');
        LoadingService.show();
        Http.post('/limparAnexoModeloDocumentoProposta', {
            row,
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public dialogDeletarAnexoProposta({ event } : IEventParam<TekGrid>) {
        event?.preventDefault();
        DialogService.show({
            name: 'dialog-deletar-anexo-documento-proposta',
            type: 'warning',
            title: 'Atenção',
            text: 'Deseja realmente excluir todos os registros selecionados? Seu documento anexado será perdido.',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deletarAnexoDocumentoProposta();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deletarAnexoDocumentoProposta() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-anexos-documento-proposta');
        LoadingService.show();
        Http.post('/deletarAnexoDocumentoProposta', grid.selectedRows).then(() => {
            grid.selectedRows = [];
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public dialogAtualizarAnexoModeloDocumentoProposta() {
        DialogService.show({
            name: 'dialog-atualizar-anexo-proposta',
            type: 'warning',
            title: 'Atenção',
            text: 'Deseja realmente sobrescrever o documento em anexo para o mesmo documento do Modelo de Documento e Idioma informados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.atualizarAnexoModeloDocumentoProposta();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public async atualizarAnexoModeloDocumentoProposta() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-anexos-documento-proposta');
        const params = {
            NRSEQMODDOCFCT: this.gridDocumentoProposta.currentRow.NRSEQMODDOCFCT,
            NRSEQORCADOC: `_${this.gridDocumentoProposta.currentRow.NRSEQORCADOC}_`,
        };
        LoadingService.show();
        Http.post('/atualizarAnexoModeloDocumentoProposta', {
            params,
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
